/***************************************************************************
 *   Copyright (C) 2005 by Christian Muehlhaeuser, Last.fm Ltd.            *
 *   chris@last.fm                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include <QtGui>

#include "playlist.h"
#include "settings.h"
#include "stationdialog.h"
#include "webserviceconnector.h"


StationDialog::StationDialog( QWidget *parent )
    : QDialog( parent )
{
    if ( parent == 0 )
        return;

    ui.setupUi( this );
    ui.customSearchBox->setImage( QApplication::applicationDirPath() + "/data/watermarks/radio.png" );
    ui.searchBox->setImage( QApplication::applicationDirPath() + "/data/watermarks/search.png" );

    menu = new QMenu( this );
    deleteAction = new QAction( tr( "&Delete item" ), this );
    resetAction  = new QAction( tr( "&Reset counter" ), this );
    clearAction  = new QAction( tr( "&Clear history" ), this );

    menu->addAction( deleteAction );
    menu->addAction( resetAction );
    menu->addSeparator();
    menu->addAction( clearAction );

    QStringList headerLabels;
    headerLabels.append( "Station" );
    headerLabels.append( "Listens" );

    ui.historyTree->setHeaderLabels( headerLabels );
    ui.historyTree->setRootIsDecorated( true );
    ui.historyTree->header()->resizeSection( 0, width() - 132 );
    ui.historyTree->headerItem()->setTextAlignment( 1, Qt::AlignHCenter );

//     ui.listenButton->setEnabled( false );
//     ui.resultBox->setEnabled( false );
//     ui.resultLabel->setVisible( true );
//     ui.resultList->setVisible( false );

    connect( deleteAction, SIGNAL( triggered() ), this, SLOT( deleteItem() ) );
    connect( resetAction, SIGNAL( triggered() ), this, SLOT( resetItem() ) );
    connect( clearAction, SIGNAL( triggered() ), this, SLOT( clearItems() ) );

    connect( ui.historyTree, SIGNAL( itemDoubleClicked( QTreeWidgetItem*, int ) ), this, SLOT( changeStationThreaded( QTreeWidgetItem*, int ) ) );
    connect( ui.historyTree, SIGNAL( customContextMenuRequested ( QPoint ) ), this, SLOT( historyMenu( QPoint ) ) );
    connect( ui.searchCombo, SIGNAL( activated( int ) ), this, SLOT( searchModeChanged( int ) ) );
//     connect( ui.listenButton, SIGNAL( clicked() ), this, SLOT( searchResultStation() ) );
    connect( ui.searchButton, SIGNAL( clicked() ), this, SLOT( search() ) );
//     connect( ui.websiteButton, SIGNAL( clicked() ), this, SLOT( websiteSearch() ) );
    connect( ui.playlistRemoveButton, SIGNAL( clicked() ), this, SLOT( playlistRemove() ) );
    connect( ui.playlistClearButton, SIGNAL( clicked() ), this, SLOT( playlistClear() ) );
    connect( ui.tagsList, SIGNAL( itemDoubleClicked( QListWidgetItem* ) ), this, SLOT( changeStationThreaded( QListWidgetItem* ) ) );

    connect( ui.personalButton, SIGNAL( clicked() ), this, SLOT( personalStation() ) );
    connect( ui.neighbourButton, SIGNAL( clicked() ), this, SLOT( neighbourStation() ) );
    connect( ui.lovedButton, SIGNAL( clicked() ), this, SLOT( lovedStation() ) );

    connect( WebserviceConnector::instance(), SIGNAL( searchArtistResult( QString, bool, QStringList ) ), this, SLOT( searchArtistResult( QString, bool, QStringList ) ) );
    connect( WebserviceConnector::instance(), SIGNAL( searchTagResult( QStringList ) ), this, SLOT( searchTagResult( QStringList ) ) );
    connect( WebserviceConnector::instance(), SIGNAL( playlistResult( Playlist ) ), this, SLOT( playlistResult( Playlist ) ) );
    connect( WebserviceConnector::instance(), SIGNAL( changeStationPrepared() ), this, SLOT( changeStationPrepared() ) );
    connect( WebserviceConnector::instance(), SIGNAL( tagsForUserResult( QStringList ) ), this, SLOT( tagsForUserResult( QStringList ) ) );
}


void
StationDialog::updateInformation()
{
    updateHistory();

    ui.personalButton->setEnabled( WebserviceConnector::instance()->isSubscriber() );
    ui.lovedButton->setEnabled( WebserviceConnector::instance()->isSubscriber() );
    ui.tagsList->clear();

    if ( WebserviceConnector::instance()->isSubscriber() )
        WebserviceConnector::instance()->tagsForUser( Settings::instance()->username() );
}


void
StationDialog::updateHistory()
{
    ui.historyTree->clear();

    QTreeWidgetItem *personal = new QTreeWidgetItem( ui.historyTree, 0 );
    personal->setText( 0, "Personal Stations" );

    QTreeWidgetItem *profile = new QTreeWidgetItem( ui.historyTree, 0 );
    profile->setText( 0, "Neighbour Stations" );

    QTreeWidgetItem *loved = new QTreeWidgetItem( ui.historyTree, 0 );
    loved->setText( 0, "Loved Tracks Stations" );

    QTreeWidgetItem *groups = new QTreeWidgetItem( ui.historyTree, 0 );
    groups->setText( 0, "Group Stations" );

    QTreeWidgetItem *similar = new QTreeWidgetItem( ui.historyTree, 0 );
    similar->setText( 0, "Similar Artist Stations" );

    QTreeWidgetItem *tags = new QTreeWidgetItem( ui.historyTree, 0 );
    tags->setText( 0, "Global Tag Stations" );

    QTreeWidgetItem *usertags = new QTreeWidgetItem( ui.historyTree, 0 );
    usertags->setText( 0, "Personal Tag Stations" );

    QTreeWidgetItem *custom = new QTreeWidgetItem( ui.historyTree, 0 );
    custom->setText( 0, "Custom Stations" );

    QStringList stations = Settings::instance()->stationItems();
    for ( int i = 0; i < stations.size(); i++ )
    {
        QTreeWidgetItem *parent = 0;
        QStringList parts = stations.at( i ).split( "/", QString::SkipEmptyParts );
        QString group = parts.at( 0 );

        if ( parts.size() > 2 && group != "usertags" )
            group = parts.at( 2 );

        if ( group == "similarartists" ) parent = similar;
        if ( group == "personal" )       parent = personal;
        if ( group == "neighbours" )     parent = profile;
        if ( group == "group" )          parent = groups;
        if ( group == "globaltags" )     parent = tags;
        if ( group == "usertags" )       parent = usertags;
        if ( group == "loved" )          parent = loved;
        if ( group == "artists" )        parent = custom;

        if ( parent )
        {
            QString stationName = Settings::instance()->stationName( stations.at( i ) );
            if ( parent != custom || !stationName.isEmpty() )
            {
                if ( stationName.isEmpty() )
                {
                    if ( group == "usertags" )
                        stationName = QUrl::fromPercentEncoding( parts.at( 2 ).toUtf8() );
                    else
                        stationName = QUrl::fromPercentEncoding( parts.at( 1 ).toUtf8() );
                }

                QTreeWidgetItem *subitem = new QTreeWidgetItem( parent, 0 );

                subitem->setText( 0, stationName );
                subitem->setText( 1, QString().setNum( Settings::instance()->stationCounter( stations.at( i ) ) ) );
                subitem->setText( 2, stations.at( i ) );
                subitem->setTextAlignment( 1, Qt::AlignHCenter );
            }
        }
    }

    ui.historyTree->setItemExpanded( similar, true );
    ui.historyTree->setItemExpanded( personal, true );
    ui.historyTree->setItemExpanded( profile, true );
    ui.historyTree->setItemExpanded( groups, true );
    ui.historyTree->setItemExpanded( tags, true );
    ui.historyTree->setItemExpanded( usertags, true );
    ui.historyTree->setItemExpanded( loved, true );
    ui.historyTree->setItemExpanded( custom, true );

    if ( similar->childCount() == 0 )  delete similar;
    if ( personal->childCount() == 0 ) delete personal;
    if ( profile->childCount() == 0 )  delete profile;
    if ( groups->childCount() == 0 )   delete groups;
    if ( tags->childCount() == 0 )     delete tags;
    if ( usertags->childCount() == 0 ) delete usertags;
    if ( loved->childCount() == 0 )    delete loved;
    if ( custom->childCount() == 0 )   delete custom;
}


void
StationDialog::tagsForUserResult( const QStringList &result )
{
    ui.tagsList->clear();

    for ( int i = 0; i < result.count(); i++ )
    {
        ui.tagsList->addItem( result[i] );
    }
}


void
StationDialog::websiteSearch()
{
    Settings::instance()->startBrowser( "http://www.last.fm/radio/" );
}


void
StationDialog::searchModeChanged( int index )
{
    switch ( index )
    {
        case 0:
            ui.infoLabel->setText( "<html><head><meta name=\"qrichtext\" content=\"1\" /></head><body style=\" white-space: pre-wrap; font-family:Sans Serif; font-weight:400; font-style:normal; text-decoration:none;\"><p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Last.fm can play radio stations by finding music similar to a given artist. Find a station by typing an artist name above.</p></body></html>" );
            ui.searchLabel->setText( "Artist:" );
            break;

        case 1:
            ui.infoLabel->setText( "<html><head><meta name=\"qrichtext\" content=\"1\" /></head><body style=\" white-space: pre-wrap; font-family:Sans Serif; font-weight:400; font-style:normal; text-decoration:none;\"><p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Last.fm can play radio stations by finding music tagged by our users. Find a station by typing a tag name above.</p></body></html>" );
            ui.searchLabel->setText( "Tag:" );
            break;
    }
}


void
StationDialog::search()
{
    ui.searchButton->setText( "Searching..." );
    ui.searchButton->setEnabled( false );

    switch ( ui.searchCombo->currentIndex() )
    {
        case 0:
            WebserviceConnector::instance()->searchArtist( ui.searchEdit->text() );
            break;

        case 1:
            WebserviceConnector::instance()->searchTag( ui.searchEdit->text() );
            break;
    }
}


void
StationDialog::searchArtistResult( const QString &artist, bool streamable, const QStringList &result )
{
    ui.searchEdit->setText( artist );
//     ui.listenButton->setEnabled( false );
//     ui.resultLabel->setVisible( true );
//     ui.resultList->setVisible( false );

    if ( artist.isEmpty() )
    {
//         ui.resultLabel->setText( "Artist not found!" );
    }
    else
    if ( !streamable )
    {
//         ui.resultLabel->setText( "Artist not streamable!" );
    }
    else
    {
//         ui.listenButton->setEnabled( true );
//         ui.resultLabel->setVisible( true );
//         ui.resultList->setVisible( false );

        resultToken = artist;
//         ui.resultLabel->setText( artist );

        for ( int i = 0; i < result.count() && i < 10; i++ )
        {
//             ui.resultLabel->setText( ui.resultLabel->text() + ", " + result[i] );
        }
    }

    ui.searchButton->setText( "Go" );
    ui.searchButton->setEnabled( true );
//     ui.resultBox->setEnabled( true );
//     ui.resultBox->setVisible( true );
}


void
StationDialog::searchTagResult( const QStringList &result )
{
    if ( result.isEmpty() )
    {
//         ui.listenButton->setEnabled( false );
//         ui.resultLabel->setVisible( true );
//         ui.resultList->setVisible( false );
//         ui.resultLabel->setText( "No matching tags found!" );
    }
    else
    {
//         ui.listenButton->setEnabled( true );
//         ui.resultLabel->setVisible( false );
//         ui.resultList->clear();
//         ui.resultList->setVisible( true );

        for ( int i = 0; i < result.count() && i < 20; i++ )
        {
//             ui.resultList->addItem( result[i] );
        }

//         ui.resultList->setCurrentItem( ui.resultList->item( 0 ) );
//         ui.resultList->setItemSelected( ui.resultList->item( 0 ), true );
//         ui.resultLabel->setText( ui.resultLabel->text().remove( 0, 2 ) );
    }

    ui.searchButton->setText( "Go" );
    ui.searchButton->setEnabled( true );
//     ui.resultBox->setEnabled( true );
//     ui.resultBox->setVisible( true );
}


void
StationDialog::personalStation()
{
    WebserviceConnector::instance()->changeStation( "user/" + Settings::instance()->username() + "/personal" );
}


void
StationDialog::neighbourStation()
{
    WebserviceConnector::instance()->changeStation( "user/" + Settings::instance()->username() + "/neighbours" );
}


void
StationDialog::lovedStation()
{
    WebserviceConnector::instance()->changeStation( "user/" + Settings::instance()->username() + "/loved" );
}


void
StationDialog::searchResultStation()
{
    switch ( ui.searchCombo->currentIndex() )
    {
        case 0:
            WebserviceConnector::instance()->changeStation( "artist/" + QUrl( resultToken ).toEncoded() + "/similarartists" );
            break;

        case 1:
//             WebserviceConnector::instance()->changeStation( "globaltags/" + QUrl( ui.resultList->currentItem()->text() ).toEncoded() );
            break;
    }
}


void
StationDialog::playlistResult( const Playlist& playlist )
{
    ui.playList->clear();
    playlistData.first.clear();
    playlistData.second.clear();

    if ( playlist.count() )
    {
        ui.playlistRemoveButton->setEnabled( true );
        ui.playlistClearButton->setEnabled( true );
    }
    else
    {
        ui.playlistRemoveButton->setEnabled( false );
        ui.playlistClearButton->setEnabled( false );
    }

    for ( int i = 0; i < playlist.count(); i++ )
    {
        QPair<QPair<QString, QString>, QString> track = playlist.trackAt( i );

        playlistData.first.append( track.first.second );
        playlistData.second.append( track.second );
        ui.playList->addItem( track.first.first );
    }
}


void
StationDialog::playlistRemove()
{
    if ( ui.playList->currentItem() )
    {
        QString id = playlistData.first.at( ui.playList->currentRow() );

        WebserviceConnector::instance()->playlistRemove( id );
        ui.playList->takeItem( ui.playList->currentRow() );
    }

    if ( ui.playList->count() == 0 )
    {
        ui.playlistRemoveButton->setEnabled( false );
        ui.playlistClearButton->setEnabled( false );
    }
}


void
StationDialog::playlistClear()
{
    ui.playList->clear();
    WebserviceConnector::instance()->playlistClear();

    ui.playlistRemoveButton->setEnabled( false );
    ui.playlistClearButton->setEnabled( false );
}


void
StationDialog::changeStationHistory()
{
    WebserviceConnector::instance()->changeStation( ui.historyTree->currentItem()->text( 2 ) );
}


void
StationDialog::changeStationTags()
{
    WebserviceConnector::instance()->changeStation( QString( "lastfm://usertags/%1/%2" )
                                                                                   .arg( QString( QUrl( Settings::instance()->username() ).toEncoded() ) )
                                                                                   .arg( QString( QUrl( ui.tagsList->currentItem()->text() ).toEncoded() ) ) );
}


void
StationDialog::changeStationThreaded( QTreeWidgetItem *item, int column )
{
    QTimer::singleShot( 0, this, SLOT( changeStationHistory() ) );
}


void
StationDialog::changeStationThreaded( QListWidgetItem *item )
{
    QTimer::singleShot( 0, this, SLOT( changeStationTags() ) );
}


void
StationDialog::changeStationPrepared()
{
    close();
    updateHistory();
}


void
StationDialog::historyMenu( const QPoint & pos )
{
    if ( ui.historyTree->currentItem() && !ui.historyTree->currentItem()->childCount() )
        menu->exec( QCursor::pos() );
}


void
StationDialog::deleteItem()
{
    if ( ui.historyTree->currentItem() )
    {
        Settings::instance()->deleteStationCounter( ui.historyTree->currentItem()->text( 2 ) );
        delete ui.historyTree->currentItem();
    }
}


void
StationDialog::resetItem()
{
    if ( ui.historyTree->currentItem() )
    {
        Settings::instance()->resetStationCounter( ui.historyTree->currentItem()->text( 2 ) );
        ui.historyTree->currentItem()->setText( 1, "0" );
    }
}


void
StationDialog::clearItems()
{
    if ( QMessageBox::question( this, "Clear history?", "This will clear your entire station history! Are you sure?",
                                      QMessageBox::Yes, QMessageBox::No ) )
    {
        Settings::instance()->clearStationCounters();
        ui.historyTree->clear();
    }
}

